@@ -35,16 +35,61 @@
 
 #include <time.h> // for seeding
 
+#ifdef _OPENMP
+#include <omp.h>
+#endif
+
 #include "linbox/linbox-config.h"
 #include "linbox/util/debug.h"
 #include "linbox/blackbox/archetype.h"
-#include "linbox/matrix/blas-matrix.h"
+#include "linbox/matrix/dense-matrix.h"
 #include "linbox/vector/vector-domain.h"
-#include "linbox/algorithms/blas-domain.h"
+#include "linbox/matrix/matrix-domain.h"
+#include "linbox/matrix/matrix-domain.h"
+
+// #include "linbox/blackbox/triplesbb-omp.h"
+#include "linbox/matrix/sparse-matrix.h"
+#include "linbox/blackbox/pascal.h"
 
 namespace LinBox
 {
 
+//Temporary fix to deal with the fact that not all Blackboxes have applyLeft()
+template<class Field,class Block>
+class MulHelper {
+public:
+	template<class Blackbox>
+	static void mul(const Field& F,
+			Block &M1, const Blackbox &M2, const Block& M3) {
+		linbox_check( M1.rowdim() == M2.rowdim());
+		linbox_check( M2.coldim() == M3.rowdim());
+		linbox_check( M1.coldim() == M3.coldim());
+
+		MatrixDomain<Field> MD(F);
+		typename Block::ColIterator        p1 = M1.colBegin();
+		typename Block::ConstColIterator   p3 = M3.colBegin();
+
+		for (; p3 != M3.colEnd(); ++p1,++p3) {
+			M2.apply(*p1,*p3);
+		}
+	}
+
+	static void mul (const Field& F,
+			 Block &M1, const SparseMatrix<Field,SparseMatrixFormat::TPL> &M2, const Block& M3) {
+		M2.applyLeft(M1,M3);
+	}
+
+	static void mul (const Field& F,
+			 Block &M1, const SparseMatrix<Field,SparseMatrixFormat::TPL_omp> &M2, const Block& M3) {
+		M2.applyLeft(M1,M3);
+	}
+
+	static void mul (const Field& F,
+	                 Block &M1, const PascalBlackbox<Field> &M2, const Block& M3) {
+		M2.applyLeft(M1,M3);
+	}
+};
+
 #ifndef MIN
 #define MIN(a,b) ((a)<(b)?(a):(b))
 #endif
@@ -75,9 +120,10 @@ namespace LinBox
 		// Sequence constructor from a blackbox and a field
 		// cs set the size of the sequence
 		BlackboxBlockContainerBase (const Blackbox *BD, const Field &F, size_t m, size_t n, size_t seed=(size_t)time(NULL)) :
-			_field(F)  , _BB(BD), _size(BD->rowdim()/m + BD->coldim()/n +2)
-			, _nn(BD->rowdim()),  _m(m), _n(n),
-			_blockU(F,_m,_nn),_blockV(F,_nn,_n),_value(_field,m,n), _seed(seed)
+			_field(&F)  , _BB(BD), _size(BD->rowdim()/m + BD->coldim()/n +2)
+			, _nn(BD->rowdim()),  _m(m), _n(n)
+			,casenumber(0)
+			,_blockU(F,_m,_nn),_blockV(F,_nn,_n),_value(field(),m,n), _seed(seed)
 		{}
 
 
@@ -111,7 +157,8 @@ namespace LinBox
 		size_t size() const            { return _size; }
 
 		// field of the sequence
-		const Field &getField () const { return _field; }
+		const Field &field () const { return *_field; }
+		const Field &getField () const { return *_field; } // deprecated
 
 		// blackbox of the sequence
 		const Blackbox *getBB () const { return _BB; }
@@ -140,7 +187,7 @@ namespace LinBox
 		/// Members
 		//--------------
 
-		Field                        _field;
+		const Field                        *_field;
 		const Blackbox             *_BB;
 		size_t                    _size; // length of sequence
 		size_t			     _nn; // _BB order (square mat)
@@ -161,21 +208,12 @@ namespace LinBox
 		/// Initializers
 		//--------------
 
+
 		// Blackbox multiplication using apply function
-		void Mul(Block &M1, const Blackbox &M2 , const Block& M3)
+		inline void Mul(Block &M1, const Blackbox &M2 , const Block& M3)
 		{
-			linbox_check( M1.rowdim() == M2.rowdim());
-			linbox_check( M2.coldim() == M3.rowdim());
-			linbox_check( M1.coldim() == M3.coldim());
-
-			typename Block::ColIterator        p1 = M1.colBegin();
-			typename Block::ConstColIterator   p3 = M3.colBegin();
-
-			for (; p3 != M3.colEnd(); ++p1,++p3) {
-				M2.apply(*p1,*p3);
-			}
-		}
-
+                        MulHelper<Field,Block>::mul(field(),M1,M2,M3);
+                }
 
 		/// User Left and Right blocks
 		void init (const Block& U, const Block& V)
@@ -188,8 +226,8 @@ namespace LinBox
 			casenumber = 1;
 			_blockU = U;
 			_blockV = V;
-			_value = Value(_field,_m,_n);
-			BlasMatrixDomain<Field> BMD(_field);
+			_value = Value(*_field,_m,_n);
+			BlasMatrixDomain<Field> BMD(*_field);
 			BMD.mul(_value, _blockU, _blockV);
 		}
 
@@ -198,7 +236,7 @@ namespace LinBox
 		{
 			casenumber = 1;
 
-			typename Field::RandIter G(_field,0,_seed);
+			typename Field::RandIter G(*_field,0,_seed);
 			Block U (m, _BB->rowdim());
 			_blockU =U;
 			Block V(_BB->coldim(), n);
@@ -212,8 +250,8 @@ namespace LinBox
 			for (; iter_V != _blockV.End();++iter_V)
 				G.random(*iter_V);
 
-			_value = Value(_field,m,n);
-			BlasMatrixDomain<Field> BMD(_field);
+			_value = Value(*_field,m,n);
+			BlasMatrixDomain<Field> BMD(*_field);
 			BMD.mul(_value, _blockU, _blockV);
 		}
 	};
@@ -223,11 +261,10 @@ namespace LinBox
 #endif // __LINBOX_blackbox_block_container_base_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
