@@ -57,7 +57,7 @@ namespace LinBox
 		 *               options for the solver
 		 */
 		LanczosSolver (const Field &F, const LanczosTraits &traits) :
-			_traits (traits), _field (F), _randiter (F), _VD (F)
+			_traits (traits), _field (&F), _randiter (F), _VD (F)
 		{}
 
 		/** @brief Constructor with a random iterator
@@ -67,7 +67,7 @@ namespace LinBox
 		 * @param r Random iterator to use for randomization
 		 */
 		LanczosSolver (const Field &F, const LanczosTraits &traits, typename Field::RandIter r) :
-			_traits (traits), _field (F), _randiter (r), _VD (F)
+			_traits (traits), _field (&F), _randiter (r), _VD (F)
 		{}
 
 		/** Solve the linear system Ax = b.
@@ -90,6 +90,7 @@ namespace LinBox
 		template <class Blackbox>
 		Vector &solve (const Blackbox &A, Vector &x, const Vector &b);
 
+		inline const Field & field() const { return *_field; }
 	private:
 
 		// Run the Lanczos iteration and return the result. Return false
@@ -98,7 +99,7 @@ namespace LinBox
 		bool iterate (const Blackbox &A, Vector &x, const Vector &b);
 
 		const LanczosTraits &_traits;
-		const Field                       &_field;
+		const Field                       *_field;
 		typename Field::RandIter           _randiter;
 		VectorDomain<Field>                _VD;
 
