@@ -28,6 +28,7 @@
 #include "linbox/util/debug.h"
 #include "linbox/algorithms/default.h"
 #include "linbox/algorithms/rational-solver.h"
+#include "linbox/vector/blas-vector.h"
 #include <utility>
 
 namespace LinBox
@@ -50,7 +51,9 @@ namespace LinBox
 
 	protected:
 
+		typedef BlasVector<Ring>         DVect;
 		Ring        r;
+        mutable typename Ring::RandIter _gen;
 		Solver solver;
 		int threshold;
 
@@ -62,7 +65,7 @@ namespace LinBox
 		LastInvariantFactor(const Solver& _solver = Solver(),
 				    const Ring& _r =Ring(),
 				    int _threshold =DEFAULTLIFTHRESHOLD) :
-			r(_r),solver(_solver), threshold(_threshold)
+                r(_r), _gen(r),solver(_solver), threshold(_threshold)
 		{
 
 			if ( _threshold <= 1) threshold = DEFAULTLIFTHRESHOLD;
@@ -99,11 +102,12 @@ namespace LinBox
 					     const Vector& PrimeL) const
 		{
 
-			r.init(lif, 1);
+			r.assign(lif, r.one);
 			int count = 0;
 			SolverReturnStatus tmp;
 			// Storage of rational solution
-			std::vector<Integer> r_num (A. coldim()); Integer r_den;
+			DVect r_num (r,A. coldim());
+			Integer r_den;
 			//std::vector<std::pair<Integer, Integer> > result (A.coldim());
 			//typename std::vector<std::pair<Integer, Integer> >::iterator result_p;
 			// vector b, RHS, 32-bit int is good enough
@@ -111,20 +115,22 @@ namespace LinBox
 			typename std::vector<int>::iterator b_p;
 			typename Vector::const_iterator Prime_p;
 
-			Integer pri, quo, rem;
+			Integer pri, quo, rem, itmp;
 
 			for (; count < threshold; ++ count) {
 				// assign b to be a random vector
 				for (b_p = b.begin(); b_p != b.end(); ++ b_p) {
-					* b_p = rand() % 268435456 - 134217728; // may need to change to use ring's random gen.
-					// dpritcha, 2004-07-26
+// 					* b_p = rand() % 268435456 - 134217728; // may need to change to use ring's random gen.
+// 					// dpritcha, 2004-07-26
+                    _gen( itmp );
+                    * b_p = (int)itmp;
 				}
 
 				// try to solve Ax = b over Ring
 				tmp = solver.solveNonsingular(r_num, r_den, A, b);
 				// If no solution found
 				if (tmp != SS_OK) {
-					r.init (lif, 0);
+					r.assign (lif, r.zero);
 					break;
 				}
 
@@ -135,7 +141,7 @@ namespace LinBox
 				for ( Prime_p = PrimeL.begin();
 				      Prime_p != PrimeL.end();
 				      ++ Prime_p) {
-					r.init (pri, (unsigned long) *Prime_p);
+					r.init (pri, *Prime_p);
 					do {
 						r.quoRem(quo,rem,lif,pri);
 						if (r.isZero(rem)) r.assign(lif,quo);
@@ -157,12 +163,13 @@ namespace LinBox
 						   const Vector& PrimeL) const
 		{
 
-			r. init(lif, 1);
-			r. init (Bonus, 1);
+			r. assign(lif, r.one);
+			r. assign(Bonus, r.one);
 			int count = 0;
 			SolverReturnStatus tmp1, tmp2;
 			// Storage of rational solution
-			std::vector<Integer> r1_num (A. coldim()), r2_num (A. coldim()); Integer r1_den, r2_den;
+			DVect r1_num (r,A. coldim()), r2_num (r,A. coldim());
+			Integer r1_den, r2_den;
 			//std::vector<std::pair<Integer, Integer> > result (A.coldim());
 			//typename std::vector<std::pair<Integer, Integer> >::iterator result_p;
 			// vector b, RHS, 32-bit int is good enough
@@ -174,17 +181,19 @@ namespace LinBox
 			for (; count < (threshold + 1) / 2; ++ count) {
 				// assign b to be a random vector
 				for (b_p = b1. begin(); b_p != b1. end(); ++ b_p) {
-					* b_p = rand();
+// 					* b_p = rand();
+                                        *b_p = _gen.random();//(* b_p);
 				}
 				for (b_p = b2. begin(); b_p != b2. end(); ++ b_p) {
-					* b_p = rand();
+// 					* b_p = rand();
+                                        *b_p = _gen.random();//(* b_p);
 				}
 				// try to solve Ax = b1, b2 over Ring
 				tmp1 = solver. solveNonsingular(r1_num, r1_den, A, b1);
 				tmp2 = solver. solveNonsingular(r2_num, r2_den, A, b2);
 				// If no solution found
 				if ((tmp1 != SS_OK) || (tmp2 != SS_OK)){
-					r.init (lif, 0);
+					r.assign (lif, r.zero);
 					break;
 				}
 
@@ -193,18 +202,20 @@ namespace LinBox
 
 				// compute the Bonus
 				Integer g, d, a11, a12, a21, a22, l, c_bonus, c_l;
-				typename std::vector<Integer>::iterator num1_p, num2_p;
-				std::vector<Integer> r1 (A. rowdim());
-				std::vector<Integer> r2 (A. rowdim());
-				typename std::vector<Integer>::iterator r1_p, r2_p;
-				r. init (l, 0);
+				typename DVect::iterator num1_p, num2_p;
+				DVect r1 (r,A. rowdim());
+				DVect r2 (r,A. rowdim());
+				typename DVect::iterator r1_p, r2_p;
+				r. assign (l, r.zero);
 				int i;
 				for (i = 0; i < 20; ++ i) {
 					for (r1_p = r1. begin(), r2_p = r2. begin(); r1_p != r1. end(); ++ r1_p, ++ r2_p) {
-						r. init (*r1_p, rand());
-						r. init (*r2_p, rand());
+// 						r. init (*r1_p, rand());
+// 						r. init (*r2_p, rand());
+						r. init (*r1_p); _gen(*r1_p);
+						r. init (*r2_p); _gen(*r2_p);
 					}
-					r. init (a11, 0); r. init (a12, 0); r. init (a21, 0); r. init (a22, 0);
+					r. assign(a11, r.zero); r. assign (a12, r.zero); r. assign (a21, r.zero); r. assign (a22, r.zero);
 					for (r1_p = r1. begin(), num1_p = r1_num. begin(); r1_p != r1. end(); ++ r1_p, ++ num1_p)
 						r. axpyin (a11, *r1_p, *num1_p);
 					for (r1_p = r1. begin(), num2_p = r2_num. begin(); r1_p != r1. end(); ++ r1_p, ++ num2_p)
@@ -263,16 +274,18 @@ namespace LinBox
 
 			if (r_num.size()!=A. coldim()) return lif=0;
 			Integer r_den;
-			std::vector<Integer> b(A.rowdim());
-			typename std::vector<Integer>::iterator b_p;
+			DVect b(r,A.rowdim());
+			typename DVect::iterator b_p;
 			//typename Vector::const_iterator Prime_p;
 
 			Integer pri, quo, rem;
 
 			// assign b to be a random vector
 			for (b_p = b.begin(); b_p != b.end(); ++ b_p) {
-				* b_p = rand() % 268435456 - 134217728; // may need to change to use ring's random gen.
-				// dpritcha, 2004-07-26
+// 				* b_p = rand() % 268435456 - 134217728; // may need to change to use ring's random gen.
+// 				// dpritcha, 2004-07-26
+                _gen( * b_p );
+                
 			}
 			//report <<"try to solve Ax = b over Ring";
 			// try to solve Ax = b over Ring
@@ -280,7 +293,7 @@ namespace LinBox
 			tmp = solver.solveNonsingular(r_num, r_den, A, b,oldMatrix);
 			// If no solution found
 			if (tmp != SS_OK) {
-				//r.init (lif, 0);
+				//r.assign (lif, r.zero);
 				//break;
 				return lif=0;
 			}
@@ -291,7 +304,7 @@ namespace LinBox
 				Integer den,t;
 				r. lcm(den,r_den,lif);
 				r. div(t, den, r_den);
-				typename std::vector<Integer>::iterator num_p = r_num.begin();
+				typename DVect::iterator num_p = r_num.begin();
 				for (; num_p != r_num. end(); ++num_p) {
 					r. mulin(*num_p, t);
 				}
@@ -305,18 +318,20 @@ namespace LinBox
 			if (Bonus==0) Bonus=1;
 			if (r1_num.size() != r2_num.size()) return Bonus=0;
 			Integer g, d, a11, a12, a21, a22, c_bonus, l, c_l;
-			typename std::vector<Integer>::iterator num1_p, num2_p;
-			std::vector<Integer> r1 (r1_num. size());
-			std::vector<Integer> r2 (r2_num. size());
-			typename std::vector<Integer>::iterator r1_p, r2_p;
-			r. init (l, 0);
+			typename DVect::iterator num1_p, num2_p;
+			DVect r1 (r,r1_num. size());
+			DVect r2 (r,r2_num. size());
+			typename DVect::iterator r1_p, r2_p;
+			r. assign (l, r.zero);
 			int i;
 			for (i = 0; i < 20; ++ i) {
 				for (r1_p = r1. begin(), r2_p = r2. begin(); r1_p != r1. end(); ++ r1_p, ++ r2_p) {
-					r. init (*r1_p, rand());
-					r. init (*r2_p, rand());
+// 					r. init (*r1_p, rand());
+// 					r. init (*r2_p, rand());
+					r. init (*r1_p); _gen(*r1_p);
+					r. init (*r2_p); _gen(*r2_p);
 				}
-				r. init (a11, 0); r. init (a12, 0); r. init (a21, 0); r. init (a22, 0);
+				r. assign (a11, r.zero); r. assign (a12, r.zero); r. assign (a21, r.zero); r. assign (a22, r.zero);
 				for (r1_p = r1. begin(), num1_p = r1_num. begin(); r1_p != r1. end(); ++ r1_p, ++ num1_p)
 					r. axpyin (a11, *r1_p, *num1_p);
 				for (r1_p = r1. begin(), num2_p = r2_num. begin(); r1_p != r1. end(); ++ r1_p, ++ num2_p)
@@ -349,7 +364,7 @@ namespace LinBox
 		Integer& lastInvariantFactor(Integer& lif, const IMatrix& A)  const
 		{
 
-			std::vector<Integer> empty_v;
+			DVect empty_v(r);
 			lastInvariantFactor (lif, A, empty_v);
 			return lif;
 		}
@@ -360,7 +375,7 @@ namespace LinBox
 		Integer& lastInvariantFactor_Bonus(Integer& lif, Integer& Bonus, const IMatrix& A)  const
 		{
 
-			std::vector<Integer> empty_v;
+			DVect empty_v(r);
 			lastInvariantFactor_Bonus (lif, Bonus, A, empty_v);
 			return lif;
 		}
@@ -372,11 +387,10 @@ namespace LinBox
 #endif //__LINBOX_last_invariant_factor_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
