@@ -32,6 +32,7 @@
 #define __LINBOX_apply_H
 
 #include "linbox/linbox-config.h"
+#include "givaro-config.h"
 #include "linbox/integer.h"
 #include "linbox/util/debug.h"
 // #if defined(__LINBOX_field_multimod_field_H) && !defined(__LINBOX_blas_matrix_domain_H)
@@ -40,10 +41,11 @@
 #include "linbox/field/multimod-field.h"
 #include "linbox/field/hom.h"
 #include "linbox/randiter/multimod-randomprime.h"
-#include "linbox/blackbox/sparse.h"
-#include "linbox/matrix/blas-matrix.h"
+#include "linbox/matrix/sparse-matrix.h"
+#include "linbox/matrix/dense-matrix.h"
 #include "linbox/algorithms/lifting-container.h"
 #include <vector>
+#include "linbox/vector/blas-vector.h"
 
 
 #include "linbox/util/timer.h"
@@ -84,32 +86,30 @@ namespace LinBox
 	class BlasApply {
 
 	public:
-		typedef typename Domain::Element    Element;
-		typedef std::vector<Element>         Vector;
+		typedef typename Domain::Element   Element;
+		typedef BlasVector<Domain>         Vector;
 
 		BlasApply(const Domain& D) :
 			_domain(D), _MD(D)
 		{
 			_domain.characteristic(_prime);
-			_domain.init(_one,1UL);
-			_domain.init(_zero,0UL);
 		}
 
 
 		//#ifdef __LINBOX_BLAS_AVAILABLE
-		inline Vector& applyV(Vector                       &y,
+		inline Vector& applyV(Vector                        &y,
 				      const BlasMatrix<Domain>     &A,
-				      const Vector                 &x) const
+				      const Vector                  &x) const
 		{
 
 			if (( _prime > 0) && ( _prime <  67108863)) {
 
-				FFLAS::fgemv((typename Domain::Father_t) _domain, FFLAS::FflasNoTrans,
+				FFLAS::fgemv(_domain, FFLAS::FflasNoTrans,
 					      A.rowdim(), A.coldim(),
-					      _one,
+					      _domain.one,
 					      A.getPointer(), A.getStride(),
 					      &x[0],1,
-					      _zero,
+					      _domain.zero,
 					      &y[0],1);
 			}
 			else {
@@ -118,19 +118,19 @@ namespace LinBox
 			return y;
 		}
 
-		inline Vector& applyVTrans(Vector                       &y,
+		inline Vector& applyVTrans(Vector                        &y,
 					   BlasMatrix<Domain>           &A,
-					   const Vector                 &x) const
+					   const Vector                  &x) const
 		{
 
 			if (( _prime > 0) && ( _prime <  67108863)) {
 
-				FFLAS::fgemv((typename Domain::Father_t) _domain, FFLAS::FflasTrans,
+				FFLAS::fgemv(_domain, FFLAS::FflasTrans,
 					      A.rowdim(), A.coldim(),
-					      _one,
+					      _domain.one,
 					      A.getPointer(), A.getStride(),
 					      &x[0],1,
-					      _zero,
+					      _domain.zero,
 					      &y[0],1);
 			}
 			else {
@@ -158,7 +158,7 @@ namespace LinBox
 
 			integer tmp;
 			bool use_neg=false;
-			size_t maxword=0;
+			uint32_t maxword=0;
 			for (size_t i=0;i<n;++i){
 				_domain.convert(tmp,x[i]);
 				if (tmp <0)
@@ -206,8 +206,8 @@ namespace LinBox
 				    (int) maxword,(int) m,(int) n, 1,
 				    xdbl,(int) n, At_dbl, (int) m, 0, ydbl, (int) m);
 
-			delete At_dbl;
-			delete xdbl;
+			delete[] At_dbl;
+			delete[] xdbl;
 
 			size_t rclen=maxword*4+5;
 			unsigned char* combined1 = new unsigned char[m*rclen];
@@ -243,9 +243,9 @@ namespace LinBox
 				else
 					result = 0;
 
-				importWords(tmp2, (size_t)rclen, -1, 1, 0, 0, combined1+i*rclen);
+				Givaro::Protected::importWords(tmp2, (size_t)rclen, -1, 1, 0, 0, combined1+i*rclen);
 				result += tmp2;
-				importWords(tmp2, (size_t)rclen, -1, 1, 0, 0, combined2+i*rclen);
+				Givaro::Protected::importWords(tmp2, (size_t)rclen, -1, 1, 0, 0, combined2+i*rclen);
 				result += tmp2;
 
 				_domain.init(y[i], result);
@@ -259,10 +259,9 @@ namespace LinBox
 
 
 	private:
-		Domain         _domain;
-		integer    _prime;
-		Element _one,_zero;
-		MatrixDomain<Domain> _MD;
+		Domain            _domain;
+		integer            _prime;
+		MatrixDomain<Domain>  _MD;
 
 
 	};
@@ -272,8 +271,8 @@ namespace LinBox
 	class MatrixApplyDomain {
 
 	public:
-		typedef typename Domain::Element    Element;
-		typedef std::vector<Element>         Vector;
+		typedef typename Domain::Element   Element;
+		typedef BlasVector<Domain>         Vector;
 
 		MatrixApplyDomain(const Domain& D, const IMatrix &Mat) :
 			_domain(D), _matM(Mat)
@@ -322,13 +321,16 @@ namespace LinBox
 		enum ApplyChoice {Classic, MatrixQadic, VectorQadic, CRT};
 
 	public:
-		typedef typename Domain::Element    Element;
-		typedef std::vector<Element>         Vector;
-		typedef IMatrix                       Matrix;
+		typedef typename Domain::Element   Element;
+		typedef BlasVector<Domain>         Vector;
+		typedef IMatrix                      Matrix;
 
 
 		BlasMatrixApplyDomain(const Domain& D, const IMatrix &Mat) :
 			_domain(D), _matM(Mat), _MD(D), _m(Mat.rowdim()), _n(Mat.coldim())
+			,use_chunks(false),use_neg(false),chunk_size(0)
+			,num_chunks(0)
+			,chunks(NULL),vchunks(NULL)
 		{
 			_switcher= Classic;_rns=NULL;
 		}
@@ -378,7 +380,7 @@ namespace LinBox
 			// Check Qadic matrix reprentation possibility
 			LinBox::integer maxChunkVal = 1;
 			maxChunkVal <<= 53;
-			maxChunkVal /= (prime-1) * _n;
+			maxChunkVal /= (prime-1) * uint64_t(_n);
 			chunk_size = maxChunkVal.bitsize();
 			use_chunks = (chunk_size >= 16);
 			//std::cout<<"max bit= "<<maxBitSize<<" "<<maxValue.size_in_base(4)*2<<"\n";std::cout<<"max value= "<<maxValue<<"\n";
@@ -390,7 +392,7 @@ namespace LinBox
 				// Check Qadic vector representation possibility
 				maxChunkVal = 1;
 				maxChunkVal <<= 53;
-				maxChunkVal /= 2*maxValue * _n;
+				maxChunkVal /= 2*maxValue * uint64_t(_n);
 				chunk_size = maxChunkVal.bitsize();
 				use_chunks = (chunk_size >= 16);
 
@@ -422,7 +424,7 @@ namespace LinBox
 					maxBitSize+=1;
 				}
 				// compute the number of chunk
-				if (maxValue*prime*_matM.coldim() < integer("9007199254740992")){
+				if (maxValue*prime* uint32_t(_matM.coldim()) < integer("9007199254740992")){
 					num_chunks=1;
 					use_neg=false;
 				}
@@ -486,10 +488,10 @@ namespace LinBox
 			case CRT:
 				if (use_neg){
 					maxValue= maxValue<<1;
-					maxBitSize+=1;
+					// maxBitSize+=1;
 				}
-				integer a_bound= maxValue*_n+1;
-				integer b_bound= sqrt(integer("9007199254740992")/_n);	std::cout<<"max prime: "<<b_bound<<" max rns: "<<a_bound<<std::endl;
+				integer a_bound= maxValue*uint64_t(_n)+1;
+				integer b_bound= sqrt(integer("9007199254740992")/uint64_t(_n));	std::cout<<"max prime: "<<b_bound<<" max rns: "<<a_bound<<std::endl;
 				MultiModRandomPrime mmrp;
 				std::vector<integer> rns_basis = mmrp.createPrimes(b_bound, a_bound);
 				_rns = new MultiModDouble(rns_basis);
@@ -505,13 +507,13 @@ namespace LinBox
 				vchunks = new double[_n*_rns->size()];
 
 				// prepare special CRT
-				Element g, s, q, zero,one,two;
+				Element g, s, q, two;
 				_q= _rns->getCRTmodulo();
-				_domain.init(q,_q);_domain.init(zero,0UL);_domain.init(one,1UL);_domain.init(two,2UL);
-				_domain.xgcd(g, _inv_q, s, q, _prime);
-				if (_domain.compare(_inv_q, zero)<0 ) _domain.addin(_inv_q,_prime);
+				_domain.init(q,_q);_domain.init(two,int64_t(2));
+				_domain.gcd(g, _inv_q, s, q, _prime);
+				if (_domain.compare(_inv_q, _domain.zero)<0 ) _domain.addin(_inv_q,_prime);
 				_domain.mul(_pq,_prime,q);
-				_domain.sub(_h_pq,_pq, one);
+				_domain.sub(_h_pq,_pq, _domain.one);
 				_domain.divin(_h_pq, two);
 
 				break;
@@ -585,8 +587,8 @@ namespace LinBox
 						 */
 						int rclen = (int)num_chunks*2 + 5;
 
-						unsigned char* combined = new unsigned char[rc*_n*rclen];
-						memset(combined, 0, rc*_n*rclen);
+						unsigned char* combined = new unsigned char[(size_t)rc*_n*(size_t)rclen];
+						memset(combined, 0, (size_t)rc*_n*(size_t)rclen);
 
 						//order from major index to minor: combining index, component of sol'n, byte
 						//compute a product (chunk times x) for each chunk
@@ -621,7 +623,7 @@ namespace LinBox
 							for (size_t j=0; j<_n; j++) {
 								// up to 53 bits will be ored-in, to be summed later
 								unsigned char* bitDest = combined;
-								bitDest += rclen*((i % rc)*_n+j);
+								bitDest += (size_t)rclen*((i % (size_t)rc)*_n+j);
 								long long mask = static_cast<long long>(ctd[j]);
 								bitDest += 2*i;
 								*(reinterpret_cast<long long*>(bitDest) ) |= mask;
@@ -640,8 +642,8 @@ namespace LinBox
 							result = 0;
 
 							for (int j=0; j<rc; j++) {
-								unsigned char* thispos = combined + rclen*(j*_n+i);
-								importWords(tmp, (size_t)rclen, -1, 1, 0, 0, thispos);
+								unsigned char* thispos = combined + (size_t)rclen*((size_t)j*_n+i);
+								Givaro::Protected::importWords(tmp, (size_t)rclen, -1, 1, 0, 0, thispos);
 								result += tmp;
 							}
 							_domain.init(y[i], result);
@@ -649,7 +651,7 @@ namespace LinBox
 						// shift back the result
 						if (use_neg) {
 							Element acc;
-							_domain.init(acc,0);
+							_domain.assign(acc,_domain.zero);
 							for (size_t i=0;i<x.size();++i)
 								_domain.addin(acc,x[i]);
 							_domain.mulin(acc,shift);
@@ -739,7 +741,7 @@ namespace LinBox
 								*(reinterpret_cast<long long*>(BitDest) ) |= mask;
 								BitDest+=rc*chunk_byte;
 							}
-							importWords(val, (size_t)rclen, -1, 1, 0, 0, combined);
+							Givaro::Protected::importWords(val, (size_t)rclen, -1, 1, 0, 0, combined);
 							result+=val;
 						}
 						_domain.init(y[i], result);
@@ -748,7 +750,7 @@ namespace LinBox
 					// shift back the result
 					if (use_neg) {
 						Element acc;
-						_domain.init(acc,0);
+						_domain.assign(acc,_domain.zero);
 						for (size_t i=0;i<x.size();++i)
 							_domain.addin(acc,x[i]);
 						_domain.mulin(acc,shift);
@@ -859,10 +861,10 @@ namespace LinBox
 					Element y_cur, b_cur;
 					// finish crt according to b
 					for (size_t i=0;i<_m;++i){
-						_domain.rem(b_cur, b[i], _prime);
+						_domain.mod(b_cur, b[i], _prime);
 						_domain.sub(y_cur, b_cur, y[i]);//std::cout<<"(b-y): "<<y_cur<<std::endl;
 						_domain.mulin(y_cur, _inv_q);//std::cout<<"((b-y)/q): "<<y_cur<<std::endl;
-						_domain.remin(y_cur, _prime);//std::cout<<"((b-y)/q mod p): "<<y_cur<<std::endl;
+						_domain.modin(y_cur, _prime);//std::cout<<"((b-y)/q mod p): "<<y_cur<<std::endl;
 						_domain.axpyin(y[i],_q, y_cur);//std::cout<<"y+p((b-y)/q mod p): "<<y[i]<<std::endl;
 						if ( y[i] > _h_pq) y[i]-=_pq;
 					}
@@ -956,15 +958,15 @@ namespace LinBox
 
 					// cout << "rc= " << rc << ", rclen = " << rclen << endl;
 
-					unsigned char* combined = new unsigned char[rc*_m*_k*rclen];
-					memset(combined, 0, rc*_m*_k*rclen);
+					unsigned char* combined = new unsigned char[(size_t)rc*_m*_k*(size_t)rclen];
+					memset(combined, 0, (size_t)rc*_m*_k*(size_t)rclen);
 
 					//order from major index to minor: combining index, component of sol'n, byte
 
 					//compute a product (chunk times x) for each chunk
 					double* ctd = new double[_m*_k];
 
-					for (size_t i=0; i<num_chunks; i++) {
+					for (size_t i=0; i<num_chunks; i++) {//!@bug why not use FFLAS ?
 						cblas_dgemm(CblasRowMajor, CblasNoTrans, CblasNoTrans,
 							    (int) _m,(int) _k,(int) _n, 1,
 							    chunks+(_m*_n*i),(int) _n, dX, (int) _k, 0, ctd, (int) _k);
@@ -973,7 +975,7 @@ namespace LinBox
 							for (size_t j=0; j<_m*_k; j++) {
 								// up to 53 bits will be ored-in, to be summed later
 								unsigned char* bitDest = combined;
-								bitDest += rclen*((i % rc)*_m*_k+j);
+								bitDest += (unsigned char)rclen*((i % (size_t)rc)*_m*_k+j);
 								long long mask = static_cast<long long>(ctd[j]);
 								bitDest += 2*i;
 								*(reinterpret_cast<long long*>(bitDest) ) |= mask;
@@ -986,7 +988,7 @@ namespace LinBox
 						LinBox::integer result, tmp;
 						if (use_neg) {
 							result = -ctd[i];
-							result <<= (num_chunks-1)*16;
+							result <<= uint64_t((num_chunks-1)*16);
 #ifdef DEBUG_CHUNK_APPLYM
 							cout << "rcneg: " << result << endl;
 #endif
@@ -995,8 +997,8 @@ namespace LinBox
 							result = 0;
 
 						for (int j=0; j<rc; j++) {
-							unsigned char* thispos = combined + rclen*(j*_m*_k+i);
-							importWords(tmp, (size_t)rclen, -1, 1, 0, 0, thispos);
+							unsigned char* thispos = combined + (unsigned char) rclen*((size_t)j*_m*_k+i);
+							Givaro::Protected::importWords(tmp, (size_t)rclen, -1, 1, 0, 0, thispos);
 							result += tmp;
 #ifdef DEBUG_CHUNK_APPLYM
 							cout << "rc[" << j << "," << i << "]:" << tmp << endl;
@@ -1091,14 +1093,14 @@ namespace LinBox
 		n  = Mat.coldim();
 		mn = m*n;
 
-		size_t tmpsize, tmpbitsize, j;
 
 
 		if (num_chunks ==1)
 			for (size_t i=0; i<mn; ++i, ++it)
 				D.convert(*(chunks+i), *it);
-		else
+		else {
 			for (size_t i=0; i<mn; ++i, ++it) {
+				size_t tmpsize, tmpbitsize, j;
 				integer tmp;
 				double* pdbl = chunks + i;
 				D.convert(tmp, *it);
@@ -1218,6 +1220,7 @@ namespace LinBox
 #endif
 					}
 			}
+		}
 	}
 
 	/** \brief split an integer vector into a padic chunk representation
@@ -1236,13 +1239,13 @@ namespace LinBox
 		size_t mn;
 		mn = V.size();
 
-		size_t tmpsize, tmpbitsize, j;
 
 		if (num_chunks ==1)
 			for (size_t i=0; i<mn; ++i, ++it)
 				D.convert(*(chunks+i), *it);
-		else
+		else {
 			for (size_t i=0; i<mn; ++i, ++it) {
+				size_t tmpsize, tmpbitsize, j;
 				integer tmp;
 				double* pdbl = chunks + i*num_chunks;
 				D.convert(tmp, *it);
@@ -1356,6 +1359,7 @@ namespace LinBox
 #endif
 					}
 			}
+	}
 
 	}
 
@@ -1375,18 +1379,18 @@ namespace LinBox
 		size_t mn;
 		mn = V.size();
 
-		size_t tmpsize, tmpbitsize, j;
 
 		if (num_chunks ==1)
 			for (size_t i=0; i<mn; ++i, ++it)
 				D.convert(*(chunks+i), *it);
-		else
+		else {
 			for (size_t i=0; i<mn; ++i, ++it) {
+		size_t tmpsize, /*tmpbitsize,*/ j;
 				integer tmp;
 				double* pdbl = chunks + i*num_chunks;
 				D.convert(tmp, *it);
 				tmpsize    = tmp.size();
-				tmpbitsize = tmp.bitsize();
+				//tmpbitsize = tmp.bitsize();
 
 				if (tmp ==0)
 					*pdbl=0;
@@ -1396,6 +1400,7 @@ namespace LinBox
 
 #if __LINBOX_SIZEOF_LONG == 8
 						// specialization for 64bits integer limbs
+						size_t tmpbitsize = tmp.bitsize();
 						for (j=0; j<tmpsize-1; j++) {
 							*pdbl     =  double(tmp[j] & 0xFFFFFFFF);
 							*(pdbl+1) =  double(tmp[j] >> 32);
@@ -1417,6 +1422,7 @@ namespace LinBox
 						++tmp;
 #if __LINBOX_SIZEOF_LONG == 8
 						// specialization for 64bits integer limbs
+						size_t tmpbitsize = tmp.bitsize();
 						for (j=0; j<tmpsize; j++) {
 							*pdbl     = double(0xFFFFFFFF ^ ( tmp[j] & 0xFFFFFFFF));
 							*(pdbl+2) = double(0xFFFFFFFF ^ ( tmp[j] >> 32));
@@ -1454,6 +1460,7 @@ namespace LinBox
 
 					}
 			}
+	}
 
 	}
 
@@ -1504,11 +1511,10 @@ namespace LinBox
 #endif // __LINBOX_apply_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
