@@ -308,17 +308,17 @@ namespace LinBox
 	 * Class handling the characteristic polynomial  of a Matrix.
 	 * \p ContPol is either:
 	 */
-	template< class Field, class ContPol, class Matrix>
+	template< class Field, class Polynomial, class Matrix>
 	class BlasMatrixDomainCharpoly {
 	public:
-		ContPol&  operator() (const Field &F, ContPol& P, const Matrix& A) const;
+		Polynomial& operator() (const Field &F, Polynomial& P, const Matrix& A) const;
 	};
 
-	template< class Field, class Matrix, class _Vrep>
-	class BlasMatrixDomainCharpoly<Field,BlasVector<Field,_Vrep>,Matrix> {
-	public:
-		BlasVector<Field,_Vrep>&  operator() (const Field &F, BlasVector<Field,_Vrep>& P, const Matrix& A) const;
-	};
+	// template< class Field, class Matrix, class _Vrep>
+	// class BlasMatrixDomainCharpoly<Field,BlasVector<Field,_Vrep>,Matrix> {
+	// public:
+	// 	BlasVector<Field,_Vrep>&  operator() (const Field &F, BlasVector<Field,_Vrep>& P, const Matrix& A) const;
+	// };
 
 	/**
 	 *  Interface for all functionnalities provided
@@ -733,41 +733,14 @@ namespace LinBox
 			return BlasMatrixDomainMinpoly<Field, Polynomial, Matrix>()(field(),P,A);
 		}
 
-		//! characteristic polynomial computation.
-		template <class Polynomial,  class Matrix >
-		Polynomial& charpoly( Polynomial& P, const Matrix& A ) const
-		{
-
-			typedef typename Polynomial::Rep PolyRep ;
-			commentator().start ("Givaro::Modular Dense Charpoly ", "MDCharpoly");
-			std::list<PolyRep> P_list;
-			P_list.clear();
-			BlasMatrixDomainCharpoly<Field, std::list<PolyRep>, Matrix >()(field(),P_list,A);
-
-
-			PolyRep tmp(A.rowdim()+1);
-			PolyRep Pt ;
-			typename std::list<PolyRep>::iterator it = P_list.begin();
-			Pt = *(it++);
-			while( it!=P_list.end() ){
-				// Waiting for an implementation of a domain of polynomials
-				mulpoly( tmp, Pt, *it);
-				Pt = tmp;
-				//	delete &(*it);
-				++it;
-			}
-			commentator().stop ("done", NULL, "MDCharpoly");
-
-			P=Pt;
-
-			return P;
-		}
-
 		//! characteristic polynomial computation.
 		template <class Polynomial, class Matrix >
-		std::list<Polynomial>& charpoly( std::list<Polynomial>& P, const Matrix& A ) const
+		Polynomial& charpoly (Polynomial& P, const Matrix& A ) const
 		{
-			return BlasMatrixDomainCharpoly<Field, std::list<Polynomial>, Matrix >()(field(),P,A);
+			commentator().start ("Givaro::Modular Dense Charpoly ", "MDCharpoly");
+			BlasMatrixDomainCharpoly<Field, Polynomial, Matrix>()(field(),P,A);
+            commentator().stop ("done", NULL, "MDCharpoly");
+			return P;
 		}
 
 		//private:
