@@ -1,6 +1,6 @@
 /* linbox/solutions/det.h
  * Copyright (C) 2001, 2002 LinBox
- * Time-stamp: <04 Oct 11 16:42:52 Jean-Guillaume.Dumas@imag.fr>
+ * Time-stamp: <06 Dec 16 13:03:05 Jean-Guillaume.Dumas@imag.fr>
  *
  *
  * ========LICENCE========
@@ -34,12 +34,13 @@
 #include "linbox/blackbox/compose.h"
 #include "linbox/solutions/methods.h"
 #include "linbox/solutions/getentry.h"
+#include "linbox/vector/blas-vector.h"
 
-#include "linbox/matrix/blas-matrix.h"
+#include "linbox/matrix/dense-matrix.h"
 #include "linbox/algorithms/blackbox-container.h"
 #include "linbox/algorithms/blackbox-container-symmetric.h"
 #include "linbox/algorithms/massey-domain.h"
-#include "linbox/algorithms/blas-domain.h"
+#include "linbox/matrix/matrix-domain.h"
 #include "linbox/algorithms/gauss.h"
 #include "linbox/vector/vector-traits.h"
 #include "linbox/util/prime-stream.h"
@@ -207,13 +208,13 @@ namespace LinBox
 			throw LinboxError("LinBox ERROR: matrix must be square for determinant computation\n");
 
 		typedef typename Blackbox::Field Field;
-		typedef std::vector<typename Field::Element> Polynomial;
 		Field F = A.field();
+		typedef BlasVector<Field> Polynomial;
 
 		if(Meth.symmetric()) {
 			commentator().start ("Symmetric Wiedemann Determinant", "sdet");
 			linbox_check (A.coldim () == A.rowdim ());
-			Polynomial               phi;
+			Polynomial               phi(F);
 			unsigned long            deg;
 			typename Field::RandIter iter (F);
 
@@ -222,36 +223,38 @@ namespace LinBox
 			// Here there is an extra diagonal computation
 			// The probability of success is also divided by two, as
 			// diag^2 contains only squares and squares are half the total elements
-			std::vector<typename Field::Element> diag (A.coldim ());
+			BlasVector<Field> diag (A.field(),A.coldim ());
 
 			typename Field::Element pi;
 			size_t i;
 			size_t iternum = 1;
 			do {
-				F.init (pi, 1);
+				F.assign(pi, F.one);
 				for (i = 0; i < A.coldim (); i++) {
 					do iter.random (diag[i]); while (F.isZero (diag[i]));
 					F.mulin (pi, diag[i]);
 				}
 
-				Diagonal<Field> D (F, diag);
-				Compose<Blackbox,Diagonal<Field> > B0 (&A, &D);
+				Diagonal<Field> D (diag);
+				Compose<Blackbox,Diagonal<Field> > B_0 (&A, &D);
 				typedef Compose<Diagonal<Field>,Compose<Blackbox,Diagonal<Field> > > Blackbox1;
-				Blackbox1 B(&D, &B0);
+				Blackbox1 B(&D, &B_0);
 
 				BlackboxContainerSymmetric<Field, Blackbox1> TF (&B, F, iter);
 
 				MasseyDomain<Field, BlackboxContainerSymmetric<Field, Blackbox1> > WD (&TF, Meth.earlyTermThreshold ());
 
 				WD.minpoly (phi, deg);
-				//                         std::cout << "\tdet: iteration # " << iternum << "\tMinpoly deg= "
-				//                                   << phi.size() << "\n" ;
-				//                         std::cout << "[" ;
-				//                         for(typename Polynomial::const_iterator refs =  phi.begin();
-				// 			        refs != phi.end() ;
-				// 				      ++refs )
-				// 		          std::cout << (*refs) << " " ;
-				//                         std::cout << "]" << std::endl;
+#if 0
+				std::cout << "\tdet: iteration # " << iternum << "\tMinpoly deg= "
+				<< phi.size() << "\n" ;
+				std::cout << "[" ;
+				for(typename Polynomial::const_iterator refs =  phi.begin();
+				    refs != phi.end() ;
+				    ++refs )
+					std::cout << (*refs) << " " ;
+				std::cout << "]" << std::endl;
+#endif
 
 				++iternum;
 			} while ( (phi.size () < A.coldim () + 1) && ( !F.isZero (phi[0]) ) );
@@ -272,25 +275,25 @@ namespace LinBox
 			commentator().start ("Wiedemann Determinant", "wdet");
 			linbox_check (A.coldim () == A.rowdim ());
 
-			Polynomial               phi;
+			Polynomial               phi(F);
 			unsigned long            deg;
 			typename Field::RandIter iter (F);
 
 			// Precondition here to separate the eigenvalues, so that
 			// minpoly (B) = charpoly (B) with high probability
-			std::vector<typename Field::Element> diag (A.coldim ());
+			BlasVector<Field> diag (F,A.coldim ());
 
 			typename Field::Element pi;
 			size_t i;
 			size_t iternum = 1;
 			do {
-				F.init (pi, 1);
+				F.assign(pi, F.one);
 				for (i = 0; i < A.coldim (); i++) {
 					do iter.random (diag[i]); while (F.isZero (diag[i]));
 					F.mulin (pi, diag[i]);
 				}
 
-				Diagonal<Field> D (F, diag);
+				Diagonal<Field> D (diag);
 
 				Compose<Blackbox,Diagonal<Field> > B (&A, &D);
 
@@ -356,7 +359,7 @@ namespace LinBox
 		typedef typename Blackbox::Field Field;
 		commentator().start ("Sparse Elimination Determinant", "SEDet");
 		// We make a copy as these data will be destroyed
-		SparseMatrix<Field, typename LinBox::Vector<Field>::SparseSeq> A1 (A.field(), A.rowdim(), A.coldim());
+		SparseMatrix<Field, SparseMatrixFormat::SparseSeq> A1 (A.field(), A.rowdim(), A.coldim());
 		typename Blackbox::Field::Element tmp;
 		for(size_t i = 0; i < A.rowdim() ; ++i)
 			for(size_t j = 0; j < A.coldim(); ++j)
@@ -379,7 +382,7 @@ namespace LinBox
 			throw LinboxError("LinBox ERROR: matrix must be square for determinant computation\n");
 		commentator().start ("Sparse Elimination Determinant", "SEDet");
 		// We make a copy as these data will be destroyed
-		SparseMatrix<Field, typename LinBox::Vector<Field>::SparseSeq> A1 (A);
+		SparseMatrix<Field, SparseMatrixFormat::SparseSeq> A1 (A);
 		GaussDomain<Field> GD ( A.field() );
 		GD.detin (d, A1, Meth.strategy ());
 		commentator().stop ("done", NULL, "SEdet");
@@ -388,7 +391,7 @@ namespace LinBox
 
 	template <class Field>
 	typename Field::Element &detin (typename Field::Element         	&d,
-					SparseMatrix<Field, typename LinBox::Vector<Field>::SparseSeq>  &A,
+					SparseMatrix<Field, SparseMatrixFormat::SparseSeq>  &A,
 					const RingCategories::ModularTag  	&tag,
 					const Method::SparseElimination     	&Meth)
 	{
@@ -415,7 +418,7 @@ namespace LinBox
 
 	template <class Field>
 	typename Field::Element &detin (typename Field::Element         	&d,
-					SparseMatrix<Field, typename LinBox::Vector<Field>::SparseSeq>  &A,
+					SparseMatrix<Field, SparseMatrixFormat::SparseSeq>  &A,
 					const RingCategories::ModularTag  	&tag,
 					const Method::Elimination     		&Meth)
 	{
@@ -512,7 +515,7 @@ namespace LinBox
 	}
 } // end of LinBox namespace
 
-#include "linbox/field/modular.h"
+#include "linbox/ring/modular.h"
 //#include "linbox/field/givaro-zpz.h"
 
 #ifdef __LINBOX_HAVE_MPI
@@ -528,7 +531,7 @@ namespace LinBox
 #include "linbox/algorithms/cra-early-single.h"
 #include "linbox/randiter/random-prime.h"
 #include "linbox/algorithms/matrix-hom.h"
-#include <typeinfo>
+
 namespace LinBox
 {
 
@@ -576,14 +579,14 @@ namespace LinBox
 
 		//  will call regular cra if C=0
 #ifdef __LINBOX_HAVE_MPI
-		MPIChineseRemainder< EarlySingleCRA< Modular<double> > > cra(4UL, C);
+		MPIChineseRemainder< EarlySingleCRA< Givaro::Modular<double> > > cra(4UL, C);
 		cra(dd, iteration, genprime);
 		if(!C || C->rank() == 0){
 			A.field().init(d, dd); // convert the result from integer to original type
 			commentator().stop ("done", NULL, "det");
 		}
 #else
-		ChineseRemainder< EarlySingleCRA< Modular<double> > > cra(4UL);
+		ChineseRemainder< EarlySingleCRA< Givaro::Modular<double> > > cra(4UL);
 		cra(dd, iteration, genprime);
 		A.field().init(d, dd); // convert the result from integer to original type
 		commentator().stop ("done", NULL, "idet");
@@ -633,8 +636,8 @@ namespace LinBox
 		Integer num,den;
 
 		IntegerModularDet<Blackbox, MyMethod> iteration(A, Meth);
-		RandomPrimeIterator genprime( 26-(int)ceil(log((double)A.rowdim())*0.7213475205));
-		RationalRemainder2< VarPrecEarlySingleCRA< Modular<double> > > rra(4UL);
+		RandomPrimeIterator genprime( (unsigned int)( 26-(int)ceil(log((double)A.rowdim())*0.7213475205)));
+		RationalRemainder2< VarPrecEarlySingleCRA< Givaro::Modular<double> > > rra(4UL);
 
 		rra(num,den, iteration, genprime);
 
@@ -680,11 +683,10 @@ namespace LinBox
 #endif // __LINBOX_det_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
