@@ -32,7 +32,6 @@
  * @details NO DOC
  */
 
-#include <vector>
 #include <algorithm>
 
 // must fix this list...
@@ -50,6 +49,8 @@
 
 #include "linbox/algorithms/rational-cra2.h"
 #include "linbox/algorithms/varprec-cra-early-multip.h"
+#include "linbox/algorithms/block-wiedemann.h"
+#include "linbox/algorithms/coppersmith.h"
 
 #ifdef __LINBOX_HAVE_IML
 #include "linbox/util/iml_wrapper.h"
@@ -149,7 +150,7 @@ namespace LinBox
 
 	//! @internal inplace Sparse Elimination.
 	template <class Vector, class Field>
-	Vector& solvein(Vector& x, SparseMatrix<Field, typename LinBox::Vector<Field>::SparseSeq>& A, const Vector& b, const Method::SparseElimination& m)
+	Vector& solvein(Vector& x, SparseMatrix<Field, SparseMatrixFormat::SparseSeq>& A, const Vector& b, const Method::SparseElimination& m)
 	{
 		commentator().start ("Sparse Elimination Solve In Place", "sesolvein");
 		GaussDomain<Field> GD ( A.field() );
@@ -159,7 +160,7 @@ namespace LinBox
 	}
 
 	template <class Vector, class Field, class Random>
-	Vector& solvein(Vector& x, SparseMatrix<Field, typename LinBox::Vector<Field>::SparseSeq>& A, const Vector& b, const Method::SparseElimination& m, Random& generator)
+	Vector& solvein(Vector& x, SparseMatrix<Field, SparseMatrixFormat::SparseSeq>& A, const Vector& b, const Method::SparseElimination& m, Random& generator)
 	{
 		commentator().start ("Sparse Elimination Solve In Place with random solution", "sesolvein");
 		GaussDomain<Field> GD ( A.field() );
@@ -175,9 +176,9 @@ namespace LinBox
 		      const Method::SparseElimination& m)
 	{
 		typedef typename Blackbox::Field Field;
-		typedef SparseMatrix<Field, typename LinBox::Vector<Field>::SparseSeq> SparseBB;
+		typedef SparseMatrix<Field,SparseMatrixFormat::SparseSeq> SparseBB;
 		SparseBB SpA(A.field(), A.rowdim(), A.coldim());
-		MatrixHom::map(SpA, A, A.field());
+		MatrixHom::map(SpA, A);
 		return solvein(x, SpA, b, m);
 	}
 
@@ -186,9 +187,9 @@ namespace LinBox
 		      const Method::SparseElimination& m, Random& generator)
 	{
 		typedef typename Blackbox::Field Field;
-		typedef SparseMatrix<Field, typename LinBox::Vector<Field>::SparseSeq> SparseBB;
+		typedef SparseMatrix<Field,SparseMatrixFormat::SparseSeq> SparseBB;
 		SparseBB SpA(A.field(), A.rowdim(), A.coldim());
-		MatrixHom::map(SpA, A, A.field());
+		MatrixHom::map(SpA, A);
 		return solvein(x, SpA, b, generator);
 	}
 
@@ -209,7 +210,7 @@ namespace LinBox
 	Vector& solvein(Vector& x,
 			GaussDomain<GF2>::Matrix    &A,
 			const Vector& b,
-			const Method::SparseElimination& m, 
+			const Method::SparseElimination& m,
             Random& generator)
 	{
 		commentator().start ("Sparse Elimination Solve In Place over GF2", "GF2sesolvein");
@@ -259,8 +260,7 @@ namespace LinBox
 
 #if 0
 		if ( ! consistent ) {  // we will return the zero vector
-			typename Field::Element zero; A.field().init(zero, 0);
-			for (typename Vector::iterator i = x.begin(); i != x.end(); ++i) *i = zero;
+			for (typename Vector::iterator i = x.begin(); i != x.end(); ++i) *i = A.field().zero;
 		}
 		return x;
 #endif
@@ -296,10 +296,8 @@ namespace LinBox
 #if 0
 		// this should be implemented directly in left_solve
 		if ( ! consistent ) {  // we will return the zero vector
-			typename Field::Element zero;
-			A.field().init(zero, 0);
 			for (typename Vector::iterator i = x.begin(); i != x.end(); ++i)
-				*i = zero;
+				*i = A.field().zero;
 		}
 #endif
 		commentator().stop ("done", NULL, "LQUP::left_solve");
@@ -349,15 +347,15 @@ namespace LinBox
 		size_t bits = 26 -(int)ceil(log((double)A.rowdim())*0.7213475205);
 		RandomPrimeIterator genprime( bits);
 
-		RationalRemainder2< VarPrecEarlyMultipCRA< Modular<double> > > rra(3UL);//using default RR method
+		RationalRemainder2< VarPrecEarlyMultipCRA< Givaro::Modular<double> > > rra(3UL);//using default RR method
 		IntegerModularSolve<BB,Vector,MethodTraits > iteration(A, b, m);
 		integer den;
-		std::vector< integer > num(A.coldim());
+		BlasVector<Givaro::ZRing<Integer> > num(A.field(),A.coldim());
 
 		rra(num, den, iteration, genprime);
 
 		typename RatVector::iterator it_x= x.begin();
-		typename std::vector<integer>::const_iterator it_num= num.begin();
+		typename BlasVector<Givaro::ZRing<Integer> >::const_iterator it_num= num.begin();
 
 		for (; it_x != x.end(); ++it_x, ++it_num){
 			integer g = gcd( *it_num, den);
@@ -448,11 +446,11 @@ namespace LinBox
 
 		Method::Dixon mDixon(m);
 		typename Ring::Element d;
-		std::vector< typename Ring::Element> num(A.coldim());
+		BlasVector<Ring> num(A.field(),A.coldim());
 		solve (num, d, A, b, tag, mDixon);
 
 		typename RatVector::iterator it_x= x.begin();
-		typename std::vector< typename Ring::Element>::const_iterator it_num= num.begin();
+		typename BlasVector<Ring>::const_iterator it_num= num.begin();
 		integer n,den;
 		A.field().convert(den,d);
 		for (; it_x != x.end(); ++it_x, ++it_num){
@@ -514,7 +512,7 @@ namespace LinBox
 	// input matrix is a SparseMatrix (no copy)
 	template <class Vect, class Ring>
 	Vect& solve(Vect& x, typename Ring::Element &d,
-		    const SparseMatrix<Ring, typename Vector<Ring>::SparseSeq>& A,
+		    const SparseMatrix<Ring, SparseMatrixFormat::SparseSeq>& A,
 		    const Vect& b,
 		    const RingCategories::IntegerTag & tag,
 		    const Method::SparseElimination& m)
@@ -536,14 +534,15 @@ namespace LinBox
 		if ((A.coldim() != x.size()) || (A.rowdim() != b.size()))
 			throw LinboxError("LinBox ERROR: dimension of data are not compatible in system solving (solving impossible)");
 
-		commentator().start ("Padic Integer Blas-based Solving", "solving");
+		commentator().start ("Padic Integer Blas-based Solving ICI", "solving");
 
-		typedef Modular<double> Field;
+		typedef Givaro::Modular<double> Field;
 		// 0.7213475205 is an upper approximation of 1/(2log(2))
-		RandomPrimeIterator genprime( 26-(int)ceil(log((double)A.rowdim())*0.7213475205));
+		RandomPrimeIterator genprime((unsigned int)( 26-(int)ceil(log((double)A.rowdim())*0.7213475205)));
 		RationalSolver<Ring, Field, RandomPrimeIterator, DixonTraits> rsolve(A.field(), genprime);
 		SolverReturnStatus status = SS_OK;
 
+
 		// if singularity unknown and matrix is square, we try nonsingular solver
 		switch ( m.singular() ) {
 		case Specifier::SINGULARITY_UNKNOWN:
@@ -615,8 +614,7 @@ namespace LinBox
 
 		if ( status == SS_INCONSISTENT ) {
 			throw LinboxMathInconsistentSystem("Linear system is inconsistent");
-			//			typename Ring::Element zero; A.field().init(zero, 0);
-			// 			for (typename Vector::iterator i = x.begin(); i != x.end(); ++i) *i = zero;
+			// 			for (typename Vector::iterator i = x.begin(); i != x.end(); ++i) *i = A.field().zero;
 		}
 		return x;
 	}
@@ -625,7 +623,7 @@ namespace LinBox
 	*/
 	template <class Vect, class Ring>
 	Vect& solve(Vect& x, typename Ring::Element &d,
-		    const SparseMatrix<Ring, typename Vector<Ring>::SparseSeq> & A,
+		    const SparseMatrix<Ring, SparseMatrixFormat::SparseSeq> & A,
 		    const Vect& b,
 		    const RingCategories::IntegerTag tag,
 		    Method::Dixon& m)
@@ -635,17 +633,15 @@ namespace LinBox
 
 		commentator().start ("Padic Integer Sparse Elimination Solving", "solving");
 
-		typedef Modular<double> Field;
+		typedef Givaro::Modular<double> Field;
 		// 0.7213475205 is an upper approximation of 1/(2log(2))
-		RandomPrimeIterator genprime( 26-(int)ceil(log((double)A.rowdim())*0.7213475205));
+		RandomPrimeIterator genprime((unsigned int) (26-(int)ceil(log((double)A.rowdim())*0.7213475205)));
 		RationalSolver<Ring, Field, RandomPrimeIterator, SparseEliminationTraits> rsolve(A.field(), genprime);
 		SolverReturnStatus status = SS_OK;
-
 		// if singularity unknown and matrix is square, we try nonsingular solver
 		switch ( m.singular() ) {
 		case Specifier::SINGULARITY_UNKNOWN:
-			switch (A.rowdim() == A.coldim() ?
-				status=rsolve.solveNonsingular(x, d, A, b,(int)m.maxTries()) : SS_SINGULAR) {
+			switch (status=rsolve.solveNonsingular(x, d, A, b,(int)m.maxTries())) {
 			case SS_OK:
 				m.singular(Specifier::NONSINGULAR);
 				break;
@@ -716,8 +712,7 @@ namespace LinBox
 
 		if ( status == SS_INCONSISTENT ) {
 			throw LinboxMathInconsistentSystem("Linear system is inconsistent");
-			//			typename Ring::Element zero; A.field().init(zero, 0);
-			// 			for (typename Vect::iterator i = x.begin(); i != x.end(); ++i) *i = zero;
+			// 			for (typename Vect::iterator i = x.begin(); i != x.end(); ++i) *i = A.field().zero;
 		}
 		return x;
 	}
@@ -756,6 +751,39 @@ namespace LinBox
 		return x;
 	}
 
+	// Only for nonsingular system for now.
+	// may throw SolverFailed or InconsistentSystem
+	template <class Vector, class BB>
+	Vector& solve(Vector& x, const BB& A, const Vector& b,
+		      const RingCategories::ModularTag & tag,
+		      const Method::BlockWiedemann& m)
+	{
+		if ((A.coldim() != x.size()) || (A.rowdim() != b.size()))
+			throw LinboxError("LinBox ERROR: dimension of data are not compatible in system solving (solving impossible)");
+
+		// adapt to earlier signature of wiedemann solver
+		typedef BlasMatrixDomain<typename BB::Field> Context;
+		Context BMD(A.field());
+		BlockWiedemannSolver<Context> BWS(BMD);
+		BWS.solveNonSingular(x, A, b);
+		return x;
+	}
+
+	// Only for nonsingular system for now.
+	// may throw SolverFailed or InconsistentSystem
+	template <class Vector, class BB>
+	Vector& solve(Vector& x, const BB& A, const Vector& b,
+		      const RingCategories::ModularTag & tag,
+		      const Method::Coppersmith& m)
+	{
+		if ((A.coldim() != x.size()) || (A.rowdim() != b.size()))
+			throw LinboxError("LinBox ERROR: dimension of data are not compatible in system solving (solving impossible)");
+
+		// adapt to earlier signature of wiedemann solver
+		CoppersmithSolver<typename BB::Field> cs(A.field());
+		cs.solveNonsingular(x, A, b);
+		return x;
+	}
 
 	/* remark 1.  I used copy constructors when switching method types.
 	   But if the method types are (empty) child classes of a common  parent class containing
@@ -766,7 +794,7 @@ namespace LinBox
 
 
 
-#include "linbox/field/modular.h"
+#include "linbox/ring/modular.h"
 #include "linbox/algorithms/rational-cra.h"
 #include "linbox/algorithms/rational-cra-early-multip.h"
 #include "linbox/randiter/random-prime.h"
@@ -792,16 +820,10 @@ namespace LinBox
 		typename Rebind<Vector, Field>::other& operator()(typename Rebind<Vector, Field>::other& x, const Field& F) const
 		{
 			typedef typename Blackbox::template rebind<Field>::other FBlackbox;
-			FBlackbox Ap(F, A.rowdim(), A.coldim());
-			MatrixHom::map(Ap, A, F);
+			FBlackbox Ap(A, F);
 
 			typedef typename Rebind<Vector, Field>::other FVector;
-			Hom<typename Blackbox::Field, Field> hom(A.field(), F);
-			FVector Bp(B.size());
-			typename Vector::const_iterator Bit = B.begin();
-			typename FVector::iterator      Bpit = Bp.begin();
-			for( ; Bit != B.end(); ++Bit, ++Bpit)
-				hom.image (*Bpit, *Bit);
+			FVector Bp(F, B);
 
 			VectorWrapper::ensureDim (x, A.coldim());
 			return solve( x, Ap, Bp, M);
@@ -820,21 +842,21 @@ namespace LinBox
 
 		commentator().start ("Integer CRA Solve", "Isolve");
 
-		RandomPrimeIterator genprime( 26 -(int)ceil(log((double)A.rowdim())*0.7213475205));
-		//         RationalRemainder< Modular<double> > rra((double)
+		RandomPrimeIterator genprime((unsigned int)( 26 -(int)ceil(log((double)A.rowdim())*0.7213475205)));
+		//         RationalRemainder< Givaro::Modular<double> > rra((double)
 		//                                                  ( A.coldim()/2.0*log((double) A.coldim()) ) );
 
-		RationalRemainder< EarlyMultipRatCRA< Modular<double> > > rra(3UL);
+		RationalRemainder< EarlyMultipRatCRA< Givaro::Modular<double> > > rra(3UL);
 		IntegerModularSolve<BB,Vector,MyMethod> iteration(A, b, M);
 
 		// use of integer due to non genericity of rra (PG 2005-09-01)
 		Integer den;
-		std::vector< Integer > num(A.coldim());
+		BlasVector<Givaro::ZRing<Integer>> num(A.field(),A.coldim());
 		rra(num, den, iteration, genprime);
 		//rra(x, d, iteration, genprime);
 
 		typename Vector::iterator it_x= x.begin();
-		typename std::vector<Integer>::const_iterator it_num= num.begin();
+		typename BlasVector<Givaro::ZRing<Integer>>::const_iterator it_num= num.begin();
 
 		// convert the result
 		for (; it_x != x.end(); ++it_x, ++it_num)
@@ -852,7 +874,9 @@ namespace LinBox
 		      const RingCategories::IntegerTag & tag,
 		      const MyMethod& M)
 	{
-		return solveCRA(x,d,A,b,tag,M);
+                Method::Dixon mDixon(M);
+                return solve(x,d,A,b,tag,mDixon);
+		//return solveCRA(x,d,A,b,tag,M);
 	}
 
 
@@ -877,9 +901,9 @@ namespace LinBox
 #ifdef __LINBOX_HAVE_IML
 	//! IML wrapper.
 	//! @bug not recognised as template spec...
-	std::vector<PID_integer::Element>&
-	solveIML(std::vector<PID_integer::Element>& x, PID_integer::Element & d,
-			 const BlasMatrix<PID_integer>& B, const std::vector<PID_integer::Element>& b,
+	BlasVector<Givaro::ZRing<Integer>>&
+	solveIML(BlasVector<Givaro::ZRing<Integer>>& x, Givaro::ZRing<Integer>::Element & d,
+			 const BlasMatrix<Givaro::ZRing<Integer>>& B, const BlasVector<Givaro::ZRing<Integer>>& b,
 			 const Method::IML& m)
 	{
 		THIS_CODE_COMPILES_BUT_IS_NOT_TESTED; // NOT MUCH
@@ -887,18 +911,18 @@ namespace LinBox
 		case(1) : { /*  non singular */
 					  linbox_check(B.rowdim()==B.coldim());
 					  mpz_t * mp_A = REINTERP_IML_CONST(B.getPointer()) ;
-					  // reinterpret_cast<mpz_t*>(const_cast<PID_integer::Element*>((B.getPointer())));
+					  // reinterpret_cast<mpz_t*>(const_cast<Givaro::ZRing<Integer>::Element*>((B.getPointer())));
 					  //B.getConstPointer() ?
 					  mpz_t * mp_B = REINTERP_IML_CONST(&b[0]);
 					  mpz_t * mp_N = REINTERP_IML(&x[0]);
 					  mpz_t mp_D ;
 					  mpz_init(mp_D);
 				  if (!m.computeRNS()) {
-					  IML::nonsingSolvLlhsMM(IML::RightSolu,B.rowdim(),1,
+					  IML::nonsingSolvLlhsMM(IML::RightSolu,(long)B.rowdim(),1,
 								 mp_A, mp_B, mp_N, mp_D);
 				  }
 				  else {
-					  long n = B.coldim();
+					  long n = (long)B.coldim();
 					  long basislen = 1;
 					  mpz_t mp_alpha, mp_maxInter;
 					  IML::FiniteField qh;
@@ -914,10 +938,10 @@ namespace LinBox
 					  mpz_clear(mp_maxInter);
 					  mpz_clear(mp_alpha);
 					  /*  CRNS[i] = [A_11, A_12] mod basis[i] */
-					  ARNS = IML_XMALLOC(IML::Double *, basislen);
+					  ARNS = IML_XMALLOC(IML::Double *, (size_t)basislen);
 					  for (long i = 0; i < basislen; ++i)
 					  {
-						  ARNS[i] = IML_XMALLOC(IML::Double, n*n);
+						  ARNS[i] = IML_XMALLOC(IML::Double,(size_t)(n*n));
 						  for (long j = 0; j < n; ++j)
 							  for (long l = 0; l < n; l++)
 								  ARNS[i][j*n+l] = (IML::Double) mpz_fdiv_ui(mp_A[j * n + l], basis[i]);
@@ -934,7 +958,7 @@ namespace LinBox
 			  break;
 		case (2) : { /*  certified */
 				   mpz_t * mp_A = REINTERP_IML_CONST(B.getPointer()) ;
-				   // reinterpret_cast<mpz_t*>(const_cast<PID_integer::Element*>((B.getPointer())));
+				   // reinterpret_cast<mpz_t*>(const_cast<Givaro::ZRing<Integer>::Element*>((B.getPointer())));
 				   //B.getConstPointer() ?
 				   mpz_t * mp_b = REINTERP_IML_CONST(&b[0]);
 				   mpz_t * mp_N = REINTERP_IML(&x[0]);
@@ -949,14 +973,14 @@ namespace LinBox
 				   }
 				   if ( !m.reduced() ) {
 					   IML::certSolveMP(m.certificate(),
-							    B.rowdim(),B.coldim(),
+							   (long) B.rowdim(),(long)B.coldim(),
 							    mp_A, mp_b, mp_N, mp_D,
 							    mp_NZ, mp_DZ);
 				   }
 				   else {
 					   IML::certSolveRedMP(m.certificate(),
 							       m.nullcol() //NULLSPACE_COLUMN
-							       ,B.rowdim(),B.coldim(),
+							       ,(long)B.rowdim(),(long)B.coldim(),
 							       mp_A, mp_b, mp_N, mp_D,
 							       mp_NZ, mp_DZ);
 
@@ -982,10 +1006,10 @@ namespace LinBox
 
 		commentator().start ("Rational CRA Solve", "Rsolve");
 		typename BB::Field::Element den;
-		std::vector<typename BB::Field::Element > num(A.coldim());
+		BlasVector<typename BB::Field> num(A.field(),A.coldim());
 		solve (num, den, A, b, tag, M);
 		typename RatVector::iterator it_x= x.begin();
-		typename std::vector<typename BB::Field::Element>::const_iterator it_num= num.begin();
+		typename BlasVector<typename BB::Field>::const_iterator it_num= num.begin();
 		integer n,d;
 		A.field().convert(d,den);
 		for (; it_x != x.end(); ++it_x, ++it_num){
@@ -1004,18 +1028,20 @@ namespace LinBox
 		if ((A.coldim() != x.size()) || (A.rowdim() != b.size()))
 			throw LinboxError("LinBox ERROR: dimension of data are not compatible in system solving (solving impossible)");
 		commentator().start ("Rational CRA Solve", "Rsolve");
-		size_t bits = 26 -(int)ceil(log((double)A.rowdim())*0.7213475205);
+		size_t bits = (size_t)(26 -(int)ceil(log((double)A.rowdim())*0.7213475205));
 		RandomPrimeIterator genprime( (unsigned) bits);
-		RationalRemainder2< VarPrecEarlyMultipCRA< Modular<double> > > rra(3UL);//using default RR method
+		RationalRemainder< EarlyMultipRatCRA< Givaro::Modular<double> > > rra(3UL);
 		IntegerModularSolve<BB,Vector,MethodTraits > iteration(A, b, m);
-		integer den;
-		std::vector< integer > num(A.coldim());
+		Integer den;
+		Givaro::ZRing<Integer> Z ;
+		BlasVector<Givaro::ZRing<Integer>> num(Z,A.coldim());
 		rra(num, den, iteration, genprime);
-		typename RatVector::iterator it_x= x.begin();
-		typename std::vector<integer>::const_iterator it_num= num.begin();
-		for (; it_x != x.end(); ++it_x, ++it_num){
-			integer g = gcd( *it_num, den);
-			*it_x = typename RatVector::value_type(*it_num/g, den/g);
+
+		auto&& it_x= x.begin();
+		for (auto it_num:num){
+			integer g = gcd( it_num, den);
+			*it_x = typename RatVector::value_type(it_num/g, den/g);
+            ++it_x;
 		}
 		commentator().stop ("done", NULL, "Rsolve");
 		return x;
@@ -1029,18 +1055,20 @@ namespace LinBox
 		if ((A.coldim() != x.size()) || (A.rowdim() != b.size()))
 			throw LinboxError("LinBox ERROR: dimension of data are not compatible in system solving (solving impossible)");
 		commentator().start ("Rational CRA Solve", "Rsolve");
-		size_t bits = 26 -(int)ceil(log((double)A.rowdim())*0.7213475205);
+		size_t bits = (size_t)(26 -(int)ceil(log((double)A.rowdim())*0.7213475205));
 		RandomPrimeIterator genprime((unsigned) bits);
-		RationalRemainder2< VarPrecEarlyMultipCRA< Modular<double> > > rra(3UL);//using default RR method
+		RationalRemainder< EarlyMultipRatCRA< Givaro::Modular<double> > > rra(3UL);
 		IntegerModularSolve<BB,RatVector,MethodTraits > iteration(A, b, m);
-		integer den;
-		std::vector< integer > num(A.coldim());
+		Integer den;
+		Givaro::ZRing<Integer> Z;
+		BlasVector<Givaro::ZRing<Integer>> num(Z,A.coldim());
 		rra(num, den, iteration, genprime);
-		typename RatVector::iterator it_x= x.begin();
-		typename std::vector<integer>::const_iterator it_num= num.begin();
-		for (; it_x != x.end(); ++it_x, ++it_num){
-			integer g = gcd( *it_num, den);
-			*it_x = typename RatVector::value_type(*it_num/g, den/g);
+
+		auto&& it_x= x.begin();
+		for (auto it_num:num){
+			integer g = gcd( it_num, den);
+			*it_x = typename RatVector::value_type(it_num/g, den/g);
+            ++it_x;
 		}
 		commentator().stop ("done", NULL, "Rsolve");
 		return x;
@@ -1049,14 +1077,14 @@ namespace LinBox
 } // LinBox
 
 #include "linbox/config-blas.h"
-#ifdef __LINBOX_HAVE_LAPACK
+#ifdef __LINBOX_HAVE_CLAPACK
 #include "linbox/algorithms/numeric-solver-lapack.h"
 #include "linbox/algorithms/rational-solver-sn.h"
 namespace LinBox {
-	std::vector<PID_integer::Element>&
-	solveNum(std::vector<PID_integer::Element>& x, PID_integer::Element & d,
-		 const BlasMatrix<PID_integer>& B, const std::vector<PID_integer::Element>& b,
-		 const Method::Numerical & m)
+	BlasVector<Givaro::ZRing<Integer>>&
+	solveNum(BlasVector<Givaro::ZRing<Integer>>& x, Givaro::ZRing<Integer>::Element & d,
+		 const BlasMatrix<Givaro::ZRing<Integer>>& B, const BlasVector<Givaro::ZRing<Integer>>& b,
+		 const Method::NumSymOverlap & m)
 	{
 		THIS_CODE_COMPILES_BUT_IS_NOT_TESTED; // NOT MUCH
 
@@ -1066,7 +1094,8 @@ namespace LinBox {
 		typedef LPS<FMatrix > NumSolver;
 		NumSolver numSolver;
 		bool e = false ;
-		RationalSolverSN<PID_integer, NumSolver > rsolver(PID_integer(), numSolver, e);
+		Givaro::ZRing<Integer> Z;
+		RationalSolverSN<Givaro::ZRing<Integer>, NumSolver > rsolver(Z, numSolver, e);
 
 		int status = rsolver.solve(x, d, B, b);
 		if (status)
@@ -1077,21 +1106,22 @@ namespace LinBox {
 #endif
 
 namespace LinBox {
-	std::vector<PID_integer::Element>&
-	solveNum(std::vector<PID_integer::Element>& x, PID_integer::Element & d,
-		 const BlasMatrix<PID_integer>& B, const std::vector<PID_integer::Element>& b,
-		 const Method::NumericalWan & m)
+	BlasVector<Givaro::ZRing<Integer>>&
+	solveNum(BlasVector<Givaro::ZRing<Integer>>& x, Givaro::ZRing<Integer>::Element & d,
+		 const BlasMatrix<Givaro::ZRing<Integer>>& B, const BlasVector<Givaro::ZRing<Integer>>& b,
+		 const Method::NumSymNorm & m)
 	{
-		THIS_CODE_COMPILES_BUT_IS_NOT_TESTED; // NOT MUCH
+		//THIS_CODE_COMPILES_BUT_IS_NOT_TESTED; // NOT MUCH
 
-		typedef Modular<int32_t> ZField;
-		// typedef Modular<double> ZField;
-		PID_integer ZZ ;
-		RationalSolver<PID_integer, ZField, RandomPrimeIterator, WanTraits> rsolver(ZZ);
+		typedef Givaro::Modular<int32_t> ZField;
+		// typedef Givaro::Modular<double> ZField;
+		Givaro::ZRing<Integer> ZZ ;
+		RationalSolver<Givaro::ZRing<Integer>, ZField, RandomPrimeIterator, NumSymNormTraits> rsolver(ZZ);
 
 		int status = rsolver.solve(x, d, B, b);
 		if (status)
-			throw "fail" ;
+			// throw "fail" ;
+			std::cerr << "fail:" << status << std::endl;
 		return x;
 	}
 }
@@ -1100,11 +1130,10 @@ namespace LinBox {
 
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
