@@ -33,6 +33,7 @@
 #include <string>
 #include <sstream>
 #include "linbox/util/matrix-stream.h"
+#include "linbox/integer.h"
 
 #if 0
 namespace LinBox__FORMAT_MATRIX_MARKET_H
@@ -45,12 +46,14 @@ namespace LinBox__FORMAT_MATRIX_MARKET_H
 namespace LinBox
 {
 
+#if 0
 #ifndef __LINBOX_integer_H
 	class integer;
 #endif
+#endif
 
 #if 1 /* Unused */
-	bool equalCaseInsensitive(const std::string s1, const char* s2)
+	bool equalCaseInsensitive(const std::string &s1, const char* s2)
 	{
 		int len = int(s1.size());
 		int counter = 0;
@@ -97,8 +100,10 @@ namespace LinBox
 				this->ms->readWhiteSpace();
 			}
 
-			if( this->sin->eof() ) return END_OF_FILE;
-			if( !this->sin->good() ) return BAD_FORMAT;
+                        if (entriesLeft != 0) {
+                                if( this->sin->eof()) return END_OF_FILE;
+                                if( !this->sin->good()) return BAD_FORMAT;
+                        }
 
 			this->knowM = this->knowN = true;
 			currentCol = currentRow = 1;
@@ -143,7 +148,7 @@ namespace LinBox
 				if( !this->sin->good() ) return BAD_FORMAT;
 			}
 
-			if( pattern ) this->ms->getField().init(v,(integer)1);
+			if( pattern ) this->ms->getField().assign(v,this->ms->getField().one);
 			else {
 				this->ms->readWhiteSpace();
 				this->ms->getField().read(*(this->sin),v);
@@ -166,8 +171,13 @@ namespace LinBox
 			std::string st(firstLine);
 			std::stringstream stin(st);
 
-			if( stin.get() != '%' || stin.get() != '%' )
+			//bb: cppcheck complained.
+			// if( stin.get() != '%' || stin.get() != '%' ) return NO_FORMAT;
+			if( stin.get() != '%')
 				return NO_FORMAT;
+			else if (stin.get() != '%' )
+				return NO_FORMAT;
+
 			if( !stin.good() ) return NO_FORMAT;
 
 			std::string s;
@@ -211,6 +221,10 @@ namespace LinBox
 		{
 			entriesLeft = -1;
 			currentCol = currentRow = 0;
+			//bb : why not intialised ?
+			array     = false ;
+			pattern   = false ;
+			symmetric = false ;
 		}
 
 		bool isSparse() const
@@ -234,11 +248,10 @@ namespace LinBox
 #endif //__LINBOX_format_matrix_market_H
 
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
-
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
