@@ -32,20 +32,20 @@
  *
  * ------------------------------------
  *
- * 
+ *
  * ========LICENCE========
  * This file is part of the library LinBox.
- * 
+ *
  * LinBox is free software: you can redistribute it and/or modify
  * it under the terms of the  GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- * 
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- * 
+ *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
@@ -66,7 +66,7 @@ namespace LinBox
 
 
 	template<>
-	class DotProductDomain<GF2> : private virtual VectorDomainBase<GF2> {
+	class DotProductDomain<GF2> : public  VectorDomainBase<GF2> {
 	public:
 
 		typedef bool Element;
@@ -98,19 +98,21 @@ namespace LinBox
 {
 
 	template <>
-	class VectorDomain<GF2> : private virtual VectorDomainBase<GF2>, private virtual DotProductDomain<GF2> {
+	//class VectorDomain<GF2> : private virtual VectorDomainBase<GF2> , private virtual DotProductDomain<GF2> {
+        class VectorDomain<GF2> : public DotProductDomain<GF2> {
 	public:
 		typedef bool Element;
 
 		VectorDomain (const VectorDomain &VD) :
-			VectorDomainBase<GF2> (VD._field), DotProductDomain<GF2> (VD._field)
+			//VectorDomainBase<GF2> (VD.field()), DotProductDomain<GF2> (VD.field())
+                        DotProductDomain<GF2> (VD.field())
 		{}
 
 		VectorDomain &operator = (const VectorDomain &) { return *this; }
 
 		const GF2 &field () const
 		{
-			return _field;
+			return DotProductDomain<GF2>::field();
 		}
 
 		template <class Vector>
@@ -254,7 +256,8 @@ namespace LinBox
 		}
 
 		VectorDomain (const GF2 &F) :
-			VectorDomainBase<GF2> (F), DotProductDomain<GF2> (F)
+			//VectorDomainBase<GF2> (F), DotProductDomain<GF2> (F)
+                        DotProductDomain<GF2> (F)
 		{}
 
 
@@ -508,7 +511,7 @@ namespace LinBox
 			add (u, w, v);
 			copy (res, u);
 
-			return u;
+			return res;
 		}
 
 		template <class Vector1, class Vector2, class Vector3>
@@ -526,7 +529,7 @@ namespace LinBox
 			sub (u, w, v);
 			copy (res, u);
 
-			return u;
+			return res;
 		}
 	};
 
@@ -538,11 +541,11 @@ namespace LinBox
 
 #endif // __LINBOX_field_vector_domain_H
 
-// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,:0,t0,+0,=s
 // Local Variables:
 // mode: C++
 // tab-width: 8
 // indent-tabs-mode: nil
 // c-basic-offset: 8
 // End:
+// vim:sts=8:sw=8:ts=8:noet:sr:cino=>s,f0,{0,g0,(0,\:0,t0,+0,=s
 
